using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using zx.matrix;

namespace TestReceiver
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class FormMain : System.Windows.Forms.Form
	{
        private System.Windows.Forms.Panel panel3;
        private System.Windows.Forms.Button btnClose;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
        private System.Timers.Timer timerStat;
		
        // Total number of messages
        private int mMsgTotalCount = 0;
        private System.Windows.Forms.Panel panelClose;
        private System.Windows.Forms.Panel panelListText;
        private System.Windows.Forms.ListBox listTrace;
        private System.Windows.Forms.Splitter splitter1;
        private System.Windows.Forms.TextBox txtMsgView;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.Label labelConnection;
        private System.Windows.Forms.Panel panel1;
        private System.Windows.Forms.Label lblNumOfMsgSec;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Panel panel4;
        private System.Windows.Forms.Button btnReset;
        private System.Windows.Forms.Label lblNumOfMsg;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label4;
        private System.Windows.Forms.Label label5;
        private System.Windows.Forms.Label label6;
        private System.Windows.Forms.TextBox tbAvailableHosts;
        private System.Windows.Forms.Panel panel2;
        private System.Windows.Forms.Button btnConnectionApply;
        private System.Windows.Forms.TextBox tbConnectionPassword;

        private const string Version = "1.4";
        // Number of messages a second
        private int mMsgSecondCount = 0;

  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	public FormMain()
	{
    InitializeComponent();

    // Initialize the matrix
    CMatrixSubsystem.Start("TestReceiver", Version);
      
    // Subscribe for the message
    CMatrixSubsystem.Subscribe(
      new zx.matrix.CStdMessage("test"),
      new ProcessHandle(MsgHandler_Std),
      CMatrixSubsystem.ESubscriptionType.Message, false);

    // Set connected handlers
    CMatrixSubsystem.Subscribe(
      new zx.matrix.CEventConnected(), 
      new CControlMethodMessageHandler(
        new ProcessHandle(WhenConnectedEvent), labelConnection), 
      CMatrixSubsystem.ESubscriptionType.Event);
    CMatrixSubsystem.Subscribe(
      new zx.matrix.CEventDisconnected(), 
      new CControlMethodMessageHandler(new ProcessHandle(WhenDisconnectedEvent), labelConnection), 
    CMatrixSubsystem.ESubscriptionType.Event);
	}
	
	//==========================================================================
	// Connected event handlers
	private void WhenConnectedEvent(zx.matrix.CMessage aMsg, zx.matrix.CAddress aAddr)
	{
	  labelConnection.ForeColor = System.Drawing.Color.LightGreen;
	  labelConnection.Text = CMatrixSubsystem.GetMasterAddress().GetFullName();
	}
	private void WhenDisconnectedEvent(zx.matrix.CMessage aMsg, zx.matrix.CAddress aAddr)
	{
	  labelConnection.ForeColor = System.Drawing.Color.Red;
	  labelConnection.Text = "Disconnected";
	}

  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Message processor handler
  public void MsgHandler_Std(CMessage aMsg, CAddress aSender)
  {
    // Cast the message
    zx.matrix.CStdMessage msg = zx.matrix.CStdMessage.Deserialize(aMsg);
      
    lock (this)
    {
      // Protected area:
      // Fill statistic
      mMsgTotalCount++;
      mMsgSecondCount++;
      // Total number of messages
      lblNumOfMsg.Text = mMsgTotalCount.ToString();
      listTrace.Items.Add(msg.Root.FindProperty("text").DC.GetStringValue());
      if (listTrace.Items.Count > 100)
        listTrace.Items.RemoveAt(0);
      listTrace.SelectedIndex = listTrace.Items.Count - 1;
    } // lock
  }

  /// <summary>
  /// Clean up any resources being used.
  /// </summary>
  protected override void Dispose( bool disposing )
  {
	  if( disposing )
	  {
		  if (components != null) 
		  {
			  components.Dispose();
		  }
	  }
	  base.Dispose( disposing );
  }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
      System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FormMain));
      this.panelClose = new System.Windows.Forms.Panel();
      this.btnClose = new System.Windows.Forms.Button();
      this.panel3 = new System.Windows.Forms.Panel();
      this.panel2 = new System.Windows.Forms.Panel();
      this.btnConnectionApply = new System.Windows.Forms.Button();
      this.tbConnectionPassword = new System.Windows.Forms.TextBox();
      this.tbAvailableHosts = new System.Windows.Forms.TextBox();
      this.label6 = new System.Windows.Forms.Label();
      this.label5 = new System.Windows.Forms.Label();
      this.panel1 = new System.Windows.Forms.Panel();
      this.label4 = new System.Windows.Forms.Label();
      this.lblNumOfMsgSec = new System.Windows.Forms.Label();
      this.label1 = new System.Windows.Forms.Label();
      this.panel4 = new System.Windows.Forms.Panel();
      this.btnReset = new System.Windows.Forms.Button();
      this.lblNumOfMsg = new System.Windows.Forms.Label();
      this.label2 = new System.Windows.Forms.Label();
      this.labelConnection = new System.Windows.Forms.Label();
      this.label3 = new System.Windows.Forms.Label();
      this.panelListText = new System.Windows.Forms.Panel();
      this.txtMsgView = new System.Windows.Forms.TextBox();
      this.splitter1 = new System.Windows.Forms.Splitter();
      this.listTrace = new System.Windows.Forms.ListBox();
      this.timerStat = new System.Timers.Timer();
      this.panelClose.SuspendLayout();
      this.panel3.SuspendLayout();
      this.panel2.SuspendLayout();
      this.panel1.SuspendLayout();
      this.panel4.SuspendLayout();
      this.panelListText.SuspendLayout();
      ((System.ComponentModel.ISupportInitialize)(this.timerStat)).BeginInit();
      this.SuspendLayout();
      // 
      // panelClose
      // 
      this.panelClose.BackColor = System.Drawing.SystemColors.ControlDark;
      this.panelClose.Controls.Add(this.btnClose);
      this.panelClose.Dock = System.Windows.Forms.DockStyle.Bottom;
      this.panelClose.Location = new System.Drawing.Point(0, 307);
      this.panelClose.Name = "panelClose";
      this.panelClose.Size = new System.Drawing.Size(650, 58);
      this.panelClose.TabIndex = 0;
      // 
      // btnClose
      // 
      this.btnClose.Anchor = System.Windows.Forms.AnchorStyles.None;
      this.btnClose.BackColor = System.Drawing.SystemColors.Control;
      this.btnClose.Location = new System.Drawing.Point(261, 13);
      this.btnClose.Name = "btnClose";
      this.btnClose.Size = new System.Drawing.Size(120, 34);
      this.btnClose.TabIndex = 0;
      this.btnClose.Text = "Close";
      this.btnClose.UseVisualStyleBackColor = false;
      this.btnClose.Click += new System.EventHandler(this.btnClose_Click);
      // 
      // panel3
      // 
      this.panel3.BackColor = System.Drawing.SystemColors.ControlDark;
      this.panel3.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
      this.panel3.Controls.Add(this.panel2);
      this.panel3.Controls.Add(this.tbConnectionPassword);
      this.panel3.Controls.Add(this.tbAvailableHosts);
      this.panel3.Controls.Add(this.label6);
      this.panel3.Controls.Add(this.label5);
      this.panel3.Controls.Add(this.panel1);
      this.panel3.Controls.Add(this.labelConnection);
      this.panel3.Controls.Add(this.label3);
      this.panel3.Dock = System.Windows.Forms.DockStyle.Top;
      this.panel3.Location = new System.Drawing.Point(0, 0);
      this.panel3.Name = "panel3";
      this.panel3.Size = new System.Drawing.Size(650, 187);
      this.panel3.TabIndex = 2;
      // 
      // panel2
      // 
      this.panel2.Controls.Add(this.btnConnectionApply);
      this.panel2.Dock = System.Windows.Forms.DockStyle.Right;
      this.panel2.Location = new System.Drawing.Point(492, 0);
      this.panel2.Name = "panel2";
      this.panel2.Size = new System.Drawing.Size(154, 90);
      this.panel2.TabIndex = 15;
      // 
      // btnConnectionApply
      // 
      this.btnConnectionApply.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
      this.btnConnectionApply.BackColor = System.Drawing.SystemColors.Control;
      this.btnConnectionApply.Location = new System.Drawing.Point(18, 28);
      this.btnConnectionApply.Name = "btnConnectionApply";
      this.btnConnectionApply.Size = new System.Drawing.Size(120, 33);
      this.btnConnectionApply.TabIndex = 4;
      this.btnConnectionApply.Text = "Apply";
      this.btnConnectionApply.UseVisualStyleBackColor = false;
      this.btnConnectionApply.Click += new System.EventHandler(this.btnConnectionApply_Click);
      // 
      // tbConnectionPassword
      // 
      this.tbConnectionPassword.Location = new System.Drawing.Point(192, 50);
      this.tbConnectionPassword.Name = "tbConnectionPassword";
      this.tbConnectionPassword.Size = new System.Drawing.Size(205, 26);
      this.tbConnectionPassword.TabIndex = 14;
      // 
      // tbAvailableHosts
      // 
      this.tbAvailableHosts.Location = new System.Drawing.Point(192, 19);
      this.tbAvailableHosts.Name = "tbAvailableHosts";
      this.tbAvailableHosts.Size = new System.Drawing.Size(205, 26);
      this.tbAvailableHosts.TabIndex = 13;
      // 
      // label6
      // 
      this.label6.AutoSize = true;
      this.label6.ForeColor = System.Drawing.SystemColors.ActiveCaptionText;
      this.label6.Location = new System.Drawing.Point(3, 54);
      this.label6.Name = "label6";
      this.label6.Size = new System.Drawing.Size(166, 20);
      this.label6.TabIndex = 12;
      this.label6.Text = "Connection password:";
      // 
      // label5
      // 
      this.label5.AutoSize = true;
      this.label5.ForeColor = System.Drawing.SystemColors.ActiveCaptionText;
      this.label5.Location = new System.Drawing.Point(3, 26);
      this.label5.Name = "label5";
      this.label5.Size = new System.Drawing.Size(119, 20);
      this.label5.TabIndex = 11;
      this.label5.Text = "Available hosts:";
      // 
      // panel1
      // 
      this.panel1.BackColor = System.Drawing.Color.Silver;
      this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.panel1.Controls.Add(this.label4);
      this.panel1.Controls.Add(this.lblNumOfMsgSec);
      this.panel1.Controls.Add(this.label1);
      this.panel1.Controls.Add(this.panel4);
      this.panel1.Controls.Add(this.lblNumOfMsg);
      this.panel1.Controls.Add(this.label2);
      this.panel1.Dock = System.Windows.Forms.DockStyle.Bottom;
      this.panel1.Location = new System.Drawing.Point(0, 90);
      this.panel1.Name = "panel1";
      this.panel1.Size = new System.Drawing.Size(646, 93);
      this.panel1.TabIndex = 10;
      // 
      // label4
      // 
      this.label4.AutoSize = true;
      this.label4.ForeColor = System.Drawing.SystemColors.InfoText;
      this.label4.Location = new System.Drawing.Point(0, 12);
      this.label4.Name = "label4";
      this.label4.Size = new System.Drawing.Size(70, 20);
      this.label4.TabIndex = 13;
      this.label4.Text = "Statistic:";
      // 
      // lblNumOfMsgSec
      // 
      this.lblNumOfMsgSec.AutoSize = true;
      this.lblNumOfMsgSec.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
      this.lblNumOfMsgSec.ForeColor = System.Drawing.SystemColors.InfoText;
      this.lblNumOfMsgSec.Location = new System.Drawing.Point(296, 63);
      this.lblNumOfMsgSec.Name = "lblNumOfMsgSec";
      this.lblNumOfMsgSec.Size = new System.Drawing.Size(19, 20);
      this.lblNumOfMsgSec.TabIndex = 12;
      this.lblNumOfMsgSec.Text = "0";
      // 
      // label1
      // 
      this.label1.AutoSize = true;
      this.label1.ForeColor = System.Drawing.SystemColors.InfoText;
      this.label1.Location = new System.Drawing.Point(2, 63);
      this.label1.Name = "label1";
      this.label1.Size = new System.Drawing.Size(247, 20);
      this.label1.TabIndex = 11;
      this.label1.Text = "Number of messages per second:";
      // 
      // panel4
      // 
      this.panel4.Controls.Add(this.btnReset);
      this.panel4.Dock = System.Windows.Forms.DockStyle.Right;
      this.panel4.Location = new System.Drawing.Point(491, 0);
      this.panel4.Name = "panel4";
      this.panel4.Size = new System.Drawing.Size(153, 91);
      this.panel4.TabIndex = 10;
      // 
      // btnReset
      // 
      this.btnReset.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
      this.btnReset.BackColor = System.Drawing.SystemColors.Control;
      this.btnReset.Location = new System.Drawing.Point(18, 15);
      this.btnReset.Name = "btnReset";
      this.btnReset.Size = new System.Drawing.Size(120, 33);
      this.btnReset.TabIndex = 3;
      this.btnReset.Text = "Reset";
      this.btnReset.UseVisualStyleBackColor = false;
      this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
      // 
      // lblNumOfMsg
      // 
      this.lblNumOfMsg.AutoSize = true;
      this.lblNumOfMsg.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
      this.lblNumOfMsg.ForeColor = System.Drawing.SystemColors.InfoText;
      this.lblNumOfMsg.Location = new System.Drawing.Point(296, 39);
      this.lblNumOfMsg.Name = "lblNumOfMsg";
      this.lblNumOfMsg.Size = new System.Drawing.Size(19, 20);
      this.lblNumOfMsg.TabIndex = 9;
      this.lblNumOfMsg.Text = "0";
      // 
      // label2
      // 
      this.label2.AutoSize = true;
      this.label2.ForeColor = System.Drawing.SystemColors.InfoText;
      this.label2.Location = new System.Drawing.Point(0, 39);
      this.label2.Name = "label2";
      this.label2.Size = new System.Drawing.Size(201, 20);
      this.label2.TabIndex = 8;
      this.label2.Text = "Total number of messages:";
      // 
      // labelConnection
      // 
      this.labelConnection.AutoSize = true;
      this.labelConnection.Font = new System.Drawing.Font("Courier New", 9F, ((System.Drawing.FontStyle)((System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Italic))), System.Drawing.GraphicsUnit.Point, ((byte)(0)));
      this.labelConnection.ForeColor = System.Drawing.Color.Red;
      this.labelConnection.Location = new System.Drawing.Point(190, -1);
      this.labelConnection.Name = "labelConnection";
      this.labelConnection.Size = new System.Drawing.Size(141, 20);
      this.labelConnection.TabIndex = 9;
      this.labelConnection.Text = "Disconnected";
      // 
      // label3
      // 
      this.label3.AutoSize = true;
      this.label3.ForeColor = System.Drawing.SystemColors.ActiveCaptionText;
      this.label3.Location = new System.Drawing.Point(3, 0);
      this.label3.Name = "label3";
      this.label3.Size = new System.Drawing.Size(94, 20);
      this.label3.TabIndex = 8;
      this.label3.Text = "Connection:";
      // 
      // panelListText
      // 
      this.panelListText.BackColor = System.Drawing.SystemColors.Control;
      this.panelListText.Controls.Add(this.txtMsgView);
      this.panelListText.Controls.Add(this.splitter1);
      this.panelListText.Controls.Add(this.listTrace);
      this.panelListText.Dock = System.Windows.Forms.DockStyle.Fill;
      this.panelListText.Location = new System.Drawing.Point(0, 187);
      this.panelListText.Name = "panelListText";
      this.panelListText.Size = new System.Drawing.Size(650, 120);
      this.panelListText.TabIndex = 3;
      // 
      // txtMsgView
      // 
      this.txtMsgView.BackColor = System.Drawing.SystemColors.Info;
      this.txtMsgView.Dock = System.Windows.Forms.DockStyle.Fill;
      this.txtMsgView.Location = new System.Drawing.Point(0, 115);
      this.txtMsgView.Multiline = true;
      this.txtMsgView.Name = "txtMsgView";
      this.txtMsgView.ReadOnly = true;
      this.txtMsgView.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
      this.txtMsgView.Size = new System.Drawing.Size(650, 5);
      this.txtMsgView.TabIndex = 9;
      // 
      // splitter1
      // 
      this.splitter1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
      this.splitter1.Dock = System.Windows.Forms.DockStyle.Top;
      this.splitter1.Location = new System.Drawing.Point(0, 104);
      this.splitter1.Name = "splitter1";
      this.splitter1.Size = new System.Drawing.Size(650, 11);
      this.splitter1.TabIndex = 6;
      this.splitter1.TabStop = false;
      // 
      // listTrace
      // 
      this.listTrace.BackColor = System.Drawing.SystemColors.Control;
      this.listTrace.Dock = System.Windows.Forms.DockStyle.Top;
      this.listTrace.ItemHeight = 20;
      this.listTrace.Location = new System.Drawing.Point(0, 0);
      this.listTrace.Name = "listTrace";
      this.listTrace.Size = new System.Drawing.Size(650, 104);
      this.listTrace.TabIndex = 5;
      this.listTrace.SelectedIndexChanged += new System.EventHandler(this.listTrace_SelectedIndexChanged);
      // 
      // timerStat
      // 
      this.timerStat.Enabled = true;
      this.timerStat.Interval = 1000D;
      this.timerStat.SynchronizingObject = this;
      this.timerStat.Elapsed += new System.Timers.ElapsedEventHandler(this.timerStat_Elapsed);
      // 
      // FormMain
      // 
      this.AutoScaleBaseSize = new System.Drawing.Size(8, 19);
      this.ClientSize = new System.Drawing.Size(650, 365);
      this.Controls.Add(this.panelListText);
      this.Controls.Add(this.panel3);
      this.Controls.Add(this.panelClose);
      this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
      this.MaximumSize = new System.Drawing.Size(800, 877);
      this.MinimumSize = new System.Drawing.Size(589, 365);
      this.Name = "FormMain";
      this.Text = "Test Receiver (version " + Version + ")";
      this.Closing += new System.ComponentModel.CancelEventHandler(this.FormMain_Closing);
      this.panelClose.ResumeLayout(false);
      this.panel3.ResumeLayout(false);
      this.panel3.PerformLayout();
      this.panel2.ResumeLayout(false);
      this.panel1.ResumeLayout(false);
      this.panel1.PerformLayout();
      this.panel4.ResumeLayout(false);
      this.panelListText.ResumeLayout(false);
      this.panelListText.PerformLayout();
      ((System.ComponentModel.ISupportInitialize)(this.timerStat)).EndInit();
      this.ResumeLayout(false);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
          Application.Run(new FormMain());
		}

        private void FormMain_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
          // Stop the matrix
          CMatrixSubsystem.Stop();
        }

        private void btnClose_Click(object sender, System.EventArgs e)
        {
          Close();
        }

        private void listTrace_SelectedIndexChanged(object sender, System.EventArgs e)
        {
          ListBox lb = (ListBox)sender;
          txtMsgView.Text = lb.SelectedItem.ToString();
        }

        private void btnReset_Click(object sender, System.EventArgs e)
        {
          lock (this)
          {
            mMsgTotalCount = 0;
            mMsgSecondCount = 0;
            lblNumOfMsg.Text = mMsgTotalCount.ToString();
            listTrace.Items.Clear();
            txtMsgView.Text = "";
          } // lock
        }

        private void timerStat_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
          lock (this)
          {
            lblNumOfMsgSec.Text = mMsgSecondCount.ToString();
            mMsgSecondCount = 0;
          }  
        }

        private void btnConnectionApply_Click(object sender, System.EventArgs e)
        {
          // Here I lock the button to avoid of friquent clicking
          btnConnectionApply.Enabled = false;
          CMatrixSubsystem.SetAvailableHosts(tbAvailableHosts.Text);
          System.Threading.Thread.Sleep(2000);
          btnConnectionApply.Enabled = true;
        }

	}
}
