﻿using System;

namespace TestServer {
  class Program {
    static void Main(string[] args)
    {
      // Initialize the matrix (note: only one instance of the server is allowed)
      zx.matrix.CMatrixSubsystem.Start("TestServer", "1.1", zx.matrix.TComponentRunType.RUN_TYPE_SINGLE);

      // Register returning server name
      zx.matrix.CMatrixSubsystem.RegisterMethod("GetMyName", Method_GetMyName);

      // Register method sending test messages
      zx.matrix.CMatrixSubsystem.RegisterMethod("SendMessages", Method_SendMessages);

      // This is called when the component has connected to the node
      zx.matrix.CMatrixSubsystem.Subscribe(
        new zx.matrix.CEventConnected(),
        (msg, addr) =>
        {
          // Set signal "initialized"
          zx.matrix.CMatrixSubsystem.SetComponentState(zx.matrix.TComponentState.STATE_INITIALIZED);
          Console.WriteLine("The Server is ready to work.");
        },
        zx.matrix.CMatrixSubsystem.ESubscriptionType.Event);
      // This is called when the component has disconnected from the node
      zx.matrix.CMatrixSubsystem.Subscribe(
        new zx.matrix.CEventDisconnected(),
        (msg, addr) => Console.WriteLine("The Server is waiting for connection..."),
        zx.matrix.CMatrixSubsystem.ESubscriptionType.Event);
    }


    //--------------------------------------------------------------------------
    // This dummy method just returns a name of the server
    // in string output parameter "MyName"
    static bool Method_GetMyName(
      zx.pl.CPropertyList inParameters,
      zx.pl.CPropertyList outParameters,
      zx.CError outError)
    {
      outParameters.AddPropertyValue("MyName", "I am a Test Server");
      return true;
    }


    //--------------------------------------------------------------------------
    // This dummy method starts sending TestMessage N times.
    // A number of times is specified in a "long" parameter "Number"
    static bool Method_SendMessages(
      zx.pl.CPropertyList inParameters,
      zx.pl.CPropertyList outParameters,
      zx.CError outError)
    {
      // Get a number of messages to send
      var number = inParameters.GetProperty("Number").DC.GetLongValue();
      if (number <= 0 || number > 1000)
      {
        outError.Error(string.Format(
          "Invalid number of messages {0}. It has to be a number in range from 1 to 1000", number));
        return false;
      }

      // Send messages of type "test".
      // Each message contains one string field with info
      var msg = new zx.matrix.CStdMessage("TestMsg", "Test");
      var fldInfo = msg.Root.AddProperty(zx.pl.CDataContainer.TypeString, "Info");
      for (var i = 0; i < number; ++i)
      {
        fldInfo.DC.SetStringValue(string.Format("This is message {0} from {1}...", i + 1, number));
        zx.matrix.CMatrixSubsystem.Send(msg);
      } // for

      return true;
    }

  }
}
