/******************************************************************************
File:        FormComponents.cs
Project:     Monitor
Appointment: 
Author:      AR
History:
15.10.2011 little bug in column 'ConnectedTo' fixed
15.09.2009 SYS_DESC_PARAM_ConnectedTo is used instead removed SYS_DESC_PARAM_MasterHost/Port
10.04.2006 Added processing of Component type from Desc.Parameters
06.04.2006 Optimized regarding the states display
*******************************************************************************/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using zx.matrix;

namespace Monitor
{
	/// <summary>
	/// Summary description for FormComponents.
	/// </summary>
	public class FormComponents : System.Windows.Forms.Form
	{
        private System.ComponentModel.IContainer components;
        private System.Windows.Forms.Panel panelGrid;
        private System.Windows.Forms.DataGrid dataGridComponents;
        private System.Data.DataSet dataSetComponents;
        private System.Data.DataTable dataTableComponents;
        private System.Data.DataColumn dataColumnName;
        private System.Data.DataColumn dataColumnSessionID;
        private System.Data.DataColumn dataColumnState;
        private System.Data.DataColumn dataColumnStartTime;
        private System.Data.DataColumn dataColumnStopTime;
        private System.Data.DataColumn dataColumnAddress;
        private System.Data.DataColumn dataColumnType;
        private System.Data.DataColumn dataColumnVersion;
        private System.Data.DataColumn dataColumnConnectedTo;
        private System.Data.DataColumn dataColumnPath;

        // Index suplier
        private static zx.CIndexSupplier stIndexSupplier = new zx.CIndexSupplier();
        // Supplier object
        zx.CIndexInstance mIndexInstance = new zx.CIndexInstance(stIndexSupplier);
        private System.Windows.Forms.ImageList imageListStates;
		
		// Message handlers IDs for unsubscription
		System.Guid mId_CMsgSystemInfo;

    		
		//----------------------------------------------------------------------------
		class CComponentGrid : zx.Paint.CGridPainter {
		  ImageList mImages;
		  Icon mIconStarted;
		  Icon mIconStopped;
		  Icon mIconInitialized;
		  Icon mIconError;
		  Icon mIconIdle;
		  Icon mIconUnknown;
        public
          // Redefined virtual from CGridPainter
          override zx.Paint.CBrushSet GetRowPaintColors(int aRow, bool aIsCurrentRow, 
                                                                     DataGridTextBoxColumn aColumn, 
                                                                               zx.Paint.CBrushSet aBrush) 
          {
            string state = mDataGrid[aRow, 0].ToString();
            Color color = new Color(); 
            if (state == SysComponentStateNames.Started)
              color = aIsCurrentRow ? System.Drawing.Color.LightBlue : 
                                      System.Drawing.Color.MidnightBlue;
            else
              if (state == SysComponentStateNames.Initialized)
                color = aIsCurrentRow ? System.Drawing.Color.White : 
                                        System.Drawing.Color.Black;
              else
                if (state == SysComponentStateNames.Stopped)  
                  color = aIsCurrentRow ? System.Drawing.Color.LightGray : System.Drawing.Color.Gray;
                else
                  if (state == SysComponentStateNames.Error)  
                    color = aIsCurrentRow ? System.Drawing.Color.Red : System.Drawing.Color.DarkRed;
                  else  
                    if (state == SysComponentStateNames.Idle)
                      color = aIsCurrentRow ? System.Drawing.Color.Cyan : 
                                              System.Drawing.Color.DarkCyan;
                    else
                      color = aIsCurrentRow ? System.Drawing.Color.LightGoldenrodYellow : 
                                              System.Drawing.Color.Olive;
            
            return color.IsEmpty ? aBrush : new zx.Paint.CBrushSet(color, 
                                                  ((System.Drawing.SolidBrush)aBrush.BackBrush).Color);
          }
          
         public
           override void PaintCell(zx.Paint.CGridPainter.CColStyle aCell, 
                                      Graphics g, Rectangle bounds, CurrencyManager source, 
                                         int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
           {
             // Only for state column
             if (aCell.MappingName == "State")
             {
               // Select icon according to the state
               string state = mDataGrid[rowNum, 0].ToString();
               Icon icon;
               if (state == SysComponentStateNames.Started)
                 icon = mIconStarted;
               else
                 if (state == SysComponentStateNames.Stopped)  
                   icon = mIconStopped;
                 else  
                   if (state == SysComponentStateNames.Error)
                     icon = mIconError;
                   else
                     if (state == SysComponentStateNames.Initialized)
                       icon = mIconInitialized;
                     else
                       if (state == SysComponentStateNames.Idle)
                         icon = mIconIdle;  
                       else
                         icon = mIconUnknown;  

               // Text area - call original base method, shifted right making place for the icon
               Rectangle bbtext = new Rectangle(bounds.X + 17, bounds.Y, bounds.Width - 17, bounds.Height);
               base.PaintCell(aCell, g, bbtext, source, rowNum, backBrush, foreBrush, alignToRight);
               // Drive icon in the beginning 
               Rectangle bb = new Rectangle(bounds.X + 1, bounds.Y, 16, bounds.Height);
               g.FillRectangle(backBrush, bounds.X, bounds.Y, 17, bounds.Height); // fill rect by bkgr color
               g.DrawIcon(icon, bb); // draw icon
             } // if
             else
               base.PaintCell(aCell, g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
           }                               
          
        public
          CComponentGrid(DataGrid aDataGrid, ImageList aImages) : base(aDataGrid)
          {
            mImages = aImages; 
		    mIconStopped = Icon.FromHandle(((Bitmap)aImages.Images[0]).GetHicon());
		    mIconStarted = Icon.FromHandle(((Bitmap)aImages.Images[1]).GetHicon());
		    mIconInitialized = Icon.FromHandle(((Bitmap)aImages.Images[2]).GetHicon());
		    mIconError = Icon.FromHandle(((Bitmap)aImages.Images[3]).GetHicon());
		    mIconIdle = Icon.FromHandle(((Bitmap)aImages.Images[4]).GetHicon());
		    mIconUnknown = Icon.FromHandle(((Bitmap)aImages.Images[5]).GetHicon());
          }  
		}; // CComponentGrid
		//----------------------------------------------------------------------------

		public FormComponents()
    {
	    //
	    // Required for Windows Form Designer support
	    //
	    InitializeComponent();

      // Subscribe for system information event
      mId_CMsgSystemInfo = CMatrixSubsystem.Subscribe(
        new zx.matrix.CEventSysInfo(), 
        new CControlMethodMessageHandler(
            new ProcessHandle(Handler_EventSysInfoChanged), this, false),
        CMatrixSubsystem.ESubscriptionType.Event);

      // Attach a grid painter to the dataGrid
      zx.Paint.CGridPainter gp = new CComponentGrid(dataGridComponents, imageListStates);
            
      // Set initial content from the system info
      UpdateDataTable(CMatrixSubsystem.GetSystemInfo());

      // Set header            
      Text += " " + (mIndexInstance.Index + 1).ToString();

      // Locate form according to ControlSaver
      new zx.CPositionSaver(Text, this, FormMain.ControlSaver);
      // Locate grid columns
      gp.Register(Text, FormMain.ControlSaver);
    }


        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        // Updates the data table according to the system information
        private void UpdateDataTable(CPlSystemInfo aSysInfo)
        {
          lock (dataTableComponents)
          {
            // Check for new rows
            IDictionaryEnumerator iter = aSysInfo.Components.GetEnum();
            while (iter.MoveNext())
            {
              // Get a component desc
              zx.matrix.CPlComponentDesc desc = 
                                     new zx.matrix.CPlComponentDesc((zx.pl.CPropertyList)iter.Value);
                                      
              // Find an entry in the table
              string id = zx.Functions.ToCppString(desc.SessionID);
              DataRow[] rows = dataTableComponents.Select("SessionID = '" + id + "'");
              DataRow row = rows.GetLength(0) == 0 ? dataTableComponents.NewRow() : rows[0];
              row[dataColumnName] = desc.Name;
              row[dataColumnSessionID] = id;
              row[dataColumnStartTime] = zx.Functions.DateToString(desc.StartTime);
              row[dataColumnStopTime] = "";
              row[dataColumnState] = 
                          zx.matrix.SysComponentStateNames.Find((zx.matrix.TComponentState)desc.State);
              row[dataColumnAddress] = string.Format("{0}/{1}", desc.HostName, desc.Port);
              
              // Additional properties
              /// Component type
              row[dataColumnType] = 
               desc.Parameters.FindPropertyV(CPlComponentDesc.SYS_DESC_PARAM_SubType).DC.GetStringValue();
              /// Component version
              row[dataColumnVersion] = 
               desc.Parameters.FindPropertyV(CPlComponentDesc.SYS_VER_PARAM_ProductVersion).DC.GetStringValue();
              /// ConnectedTo addresses
              row[dataColumnConnectedTo] = desc.Parameters.FindPropertyV(CPlComponentDesc.SYS_DESC_PARAM_ConnectedTo).DC.GetStringValue();
              /// Application path
              row[dataColumnPath] = 
               desc.Parameters.FindPropertyV(CPlComponentDesc.SYS_DESC_PARAM_AppPath).DC.GetStringValue();

              if (rows.GetLength(0) == 0)
                // This is a new row - add it
                dataTableComponents.Rows.Add(row);
            } // while
            
            // Check for old rows
            for (int i = 0; i < dataTableComponents.Rows.Count; i++)
            {
              DataRow row = dataTableComponents.Rows[i];
              if (aSysInfo.FindComponent(zx.matrix.CPlComponentDesc.FldSessionID, 
                                         row[dataColumnSessionID].ToString()) == null)
                // Component has been removed - mark it
                if (row[dataColumnState].ToString() != SysComponentStateNames.Stopped)
                {
                  row[dataColumnStopTime] = zx.Functions.DateToString(System.DateTime.Now);
                  row[dataColumnState] = SysComponentStateNames.Stopped;
                } // if  
                
              if (row[dataColumnState].ToString() == SysComponentStateNames.Stopped)
              {
                // Remove old line
                System.DateTime d = 
                   System.DateTime.Parse(row[dataColumnStopTime].ToString().Replace(".", 
                                                                                "-").Replace(" ", "T"));
                if (System.DateTime.Now.Ticks - d.Ticks > 3*60*10000000.00) // 3min
                {
                  dataTableComponents.Rows.Remove(row);
                  i--; // stay on the same line
                } // if  
              } // if    
            } // for
          } // lock
        }
        //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        
        
        
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
			  // Free the index object
			  mIndexInstance.Dispose();
			  
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


    //==========================================================================
    private void Handler_EventSysInfoChanged(CMessage aMsg, CAddress aSender)
    {
      zx.matrix.CEventSysInfo msg = new zx.matrix.CEventSysInfo(aMsg);
      msg.Deserialize();
      UpdateDataTable(msg.SysInfo);
    }

 		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FormComponents));
            this.panelGrid = new System.Windows.Forms.Panel();
            this.dataGridComponents = new System.Windows.Forms.DataGrid();
            this.dataTableComponents = new System.Data.DataTable();
            this.dataColumnState = new System.Data.DataColumn();
            this.dataColumnName = new System.Data.DataColumn();
            this.dataColumnType = new System.Data.DataColumn();
            this.dataColumnVersion = new System.Data.DataColumn();
            this.dataColumnStartTime = new System.Data.DataColumn();
            this.dataColumnStopTime = new System.Data.DataColumn();
            this.dataColumnAddress = new System.Data.DataColumn();
            this.dataColumnConnectedTo = new System.Data.DataColumn();
            this.dataColumnPath = new System.Data.DataColumn();
            this.dataColumnSessionID = new System.Data.DataColumn();
            this.dataSetComponents = new System.Data.DataSet();
            this.imageListStates = new System.Windows.Forms.ImageList(this.components);
            this.panelGrid.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridComponents)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataTableComponents)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataSetComponents)).BeginInit();
            this.SuspendLayout();
            // 
            // panelGrid
            // 
            this.panelGrid.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelGrid.Controls.Add(this.dataGridComponents);
            this.panelGrid.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelGrid.Location = new System.Drawing.Point(0, 0);
            this.panelGrid.Name = "panelGrid";
            this.panelGrid.Size = new System.Drawing.Size(464, 429);
            this.panelGrid.TabIndex = 0;
            // 
            // dataGridComponents
            // 
            this.dataGridComponents.BackColor = System.Drawing.SystemColors.Control;
            this.dataGridComponents.BackgroundColor = System.Drawing.Color.DarkGray;
            this.dataGridComponents.CaptionVisible = false;
            this.dataGridComponents.DataMember = "";
            this.dataGridComponents.DataSource = this.dataTableComponents;
            this.dataGridComponents.Dock = System.Windows.Forms.DockStyle.Fill;
            this.dataGridComponents.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
            this.dataGridComponents.HeaderBackColor = System.Drawing.SystemColors.ControlLight;
            this.dataGridComponents.HeaderForeColor = System.Drawing.SystemColors.ControlText;
            this.dataGridComponents.Location = new System.Drawing.Point(0, 0);
            this.dataGridComponents.Name = "dataGridComponents";
            this.dataGridComponents.Size = new System.Drawing.Size(460, 425);
            this.dataGridComponents.TabIndex = 0;
            // 
            // dataTableComponents
            // 
            this.dataTableComponents.Columns.AddRange(new System.Data.DataColumn[] {
                                                                                       this.dataColumnState,
                                                                                       this.dataColumnName,
                                                                                       this.dataColumnType,
                                                                                       this.dataColumnVersion,
                                                                                       this.dataColumnStartTime,
                                                                                       this.dataColumnStopTime,
                                                                                       this.dataColumnAddress,
                                                                                       this.dataColumnConnectedTo,
                                                                                       this.dataColumnPath,
                                                                                       this.dataColumnSessionID});
            this.dataTableComponents.TableName = "Components";
            // 
            // dataColumnState
            // 
            this.dataColumnState.ColumnName = "State";
            // 
            // dataColumnName
            // 
            this.dataColumnName.ColumnName = "Name";
            // 
            // dataColumnType
            // 
            this.dataColumnType.Caption = "Type";
            this.dataColumnType.ColumnName = "Type";
            // 
            // dataColumnVersion
            // 
            this.dataColumnVersion.Caption = "Version";
            this.dataColumnVersion.ColumnName = "Version";
            // 
            // dataColumnStartTime
            // 
            this.dataColumnStartTime.Caption = "Start Time";
            this.dataColumnStartTime.ColumnName = "StartTime";
            // 
            // dataColumnStopTime
            // 
            this.dataColumnStopTime.Caption = "Stop Time";
            this.dataColumnStopTime.ColumnName = "StopTime";
            // 
            // dataColumnAddress
            // 
            this.dataColumnAddress.ColumnName = "Address";
            // 
            // dataColumnConnectedTo
            // 
            this.dataColumnConnectedTo.Caption = "Connected To";
            this.dataColumnConnectedTo.ColumnName = "ConnectedTo";
            // 
            // dataColumnPath
            // 
            this.dataColumnPath.Caption = "Path";
            this.dataColumnPath.ColumnName = "Path";
            // 
            // dataColumnSessionID
            // 
            this.dataColumnSessionID.Caption = "Session ID";
            this.dataColumnSessionID.ColumnName = "SessionID";
            // 
            // dataSetComponents
            // 
            this.dataSetComponents.DataSetName = "Components";
            this.dataSetComponents.Locale = new System.Globalization.CultureInfo("de-DE");
            this.dataSetComponents.Tables.AddRange(new System.Data.DataTable[] {
                                                                                   this.dataTableComponents});
            // 
            // imageListStates
            // 
            this.imageListStates.ImageSize = new System.Drawing.Size(16, 16);
            this.imageListStates.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListStates.ImageStream")));
            this.imageListStates.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // FormComponents
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(464, 429);
            this.Controls.Add(this.panelGrid);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.Name = "FormComponents";
            this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
            this.Text = "Components";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.FormComponents_Closing);
            this.panelGrid.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dataGridComponents)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataTableComponents)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataSetComponents)).EndInit();
            this.ResumeLayout(false);

        }
		#endregion

        private void FormComponents_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
          // Unsubscribe from the messages
          CMatrixSubsystem.Unsubscribe(mId_CMsgSystemInfo);
        }

	}
}
