using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace Monitor
{
	/// <summary>
	/// Summary description for FormLog.
	/// </summary>
	public class FormLog : System.Windows.Forms.Form
	{
        // Index suplier
        private static zx.CIndexSupplier stIndexSupplier = new zx.CIndexSupplier();
        // Supplier object
        zx.CIndexInstance mIndexInstance = new zx.CIndexInstance(stIndexSupplier);
        // Log client
        const string LOG_SERVER_NAME = "FlexTier Log Subsystem";
        zx.matrix.log.CClient mLogClient = new zx.matrix.log.CClient(LOG_SERVER_NAME);
        
		// Message handlers IDs for unsubscription
		System.Guid mId_CEventComponentComponentStateChanged;
        private System.Windows.Forms.Panel panelFilter;
        private System.Data.DataSet dataSetLog;
        private System.Data.DataTable dataTableLog;
        private System.Windows.Forms.ImageList imageListSeverities;
        private System.Data.DataColumn dataColumnTime;
        private System.Data.DataColumn dataColumnSeverity;
        private System.Data.DataColumn dataColumnID;
        private System.Data.DataColumn dataColumnHost;
        private System.Data.DataColumn dataColumnComponent;
        private System.Data.DataColumn dataColumnSession;
        private System.Data.DataColumn dataColumnText;
        private System.Data.DataColumn dataColumnRecordID;
        private System.ComponentModel.IContainer components;
        private System.Windows.Forms.Panel panelTotal;
        private System.Windows.Forms.Splitter splitter1;
        private DataGrid dataGridLog;
        private System.Windows.Forms.StatusBar statusBarLog;
        private System.Windows.Forms.TextBox textBoxLog;
        private System.Windows.Forms.Panel panelFilterButtons;
        private System.Windows.Forms.Button buttonRefresh;
        private System.Windows.Forms.StatusBarPanel statusBarPanelStatus;
        private System.Windows.Forms.StatusBarPanel statusBarPanelEvents;
        private System.Windows.Forms.ImageList imageListStates;
		private System.Windows.Forms.Panel panelFilterAll;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox textBoxFilterComponents;
		private System.Windows.Forms.TextBox textBoxFilterHosts;
		private System.Windows.Forms.TextBox textBoxFilterMessageIDs;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.CheckBox checkBoxSeverityError;
		private System.Windows.Forms.CheckBox checkBoxSeverityWarning;
		private System.Windows.Forms.CheckBox checkBoxSeverityInfo;
		private System.Windows.Forms.CheckBox checkBoxSeveritySuccess;
		private System.Windows.Forms.Label label4;
        
        // Record ID sequence, protected by dataTableLog
        int mRecordID = 0;
        
        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        // Helper - sets disconnected status
        public void SetConnectionStatus(string aMsg, int aIconIndex)
        {
          statusBarPanelStatus.Text = aMsg;  
          statusBarPanelStatus.Icon = 
                              Icon.FromHandle(((Bitmap)imageListStates.Images[aIconIndex]).GetHicon());
        }
        // Helper - opens a session and sets status bar
        void OpenSession()
        {
          zx.CError err = null;
          // Create severities parameter - must be int enumeration
          string severities = "";
          // Fill severities only if there are unchecked - it will work faster on the server side
          if (!checkBoxSeveritySuccess.Checked || 
              !checkBoxSeverityInfo.Checked ||
              !checkBoxSeverityWarning.Checked ||
              !checkBoxSeverityError.Checked)
          {    
            if (checkBoxSeveritySuccess.Checked)
              zx.Functions.AddStringPart(ref severities, 
                                       ((int)zx.matrix.log.TSeverities.LOG_SUCCESS).ToString(), ",");
            if (checkBoxSeverityInfo.Checked)
              zx.Functions.AddStringPart(ref severities, 
                                       ((int)zx.matrix.log.TSeverities.LOG_INFO).ToString(), ",");
            if (checkBoxSeverityWarning.Checked)
              zx.Functions.AddStringPart(ref severities, 
                                       ((int)zx.matrix.log.TSeverities.LOG_WARNING).ToString(), ",");
            if (checkBoxSeverityError.Checked)
              zx.Functions.AddStringPart(ref severities, 
                                       ((int)zx.matrix.log.TSeverities.LOG_ERROR).ToString(), ",");
          } // if                                       
          
          if (!mLogClient.OpenSession(textBoxFilterMessageIDs.Text, 
                                      severities,
                                      textBoxFilterComponents.Text,
                                      textBoxFilterHosts.Text,
                                      ref err))
            SetConnectionStatus(err.ErrorMessage, 1);
          else
            SetConnectionStatus("Connected", 0);
        }
        //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		
		//----------------------------------------------------------------------------
		// Log grid painter
		//----------------------------------------------------------------------------
		class CLogGrid : zx.Paint.CGridPainter {
          ImageList mImages;
          // Icons, corresponding to the log severities
          Icon mIconInfo;
          Icon mIconSuccess;
          Icon mIconWarning;
          Icon mIconError;
          Icon mIconUnknown;
          // Reference to the data table
          System.Data.DataTable mTable;
          // Reference to the log client
          zx.matrix.log.CClient mLogClient;
          // Reference to the form
          FormLog mFormLog;
        public
          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
          override zx.Paint.CBrushSet GetRowPaintColors(int aRow, bool aIsCurrentRow, 
                                                                     DataGridTextBoxColumn aColumn, 
                                                                               zx.Paint.CBrushSet aBrush) 
          {
            // Be sure that record is in the buffer
            SupplyRecord(aRow);
            
            // Set color
            Color fore_color = new Color(); 
            Color back_color = new Color(); 
            string severity = mDataGrid[aRow, 0].ToString(); // the first field
            if (severity == zx.matrix.log.CClient.SEVERITY_Success)  
            {
              if (aIsCurrentRow)
                fore_color = System.Drawing.Color.LightBlue;
              else  
                back_color = System.Drawing.Color.LightBlue;
            }  
            else  
              if (severity == zx.matrix.log.CClient.SEVERITY_Warning)
              {
                if (aIsCurrentRow)
                  fore_color = System.Drawing.Color.LightGoldenrodYellow;
                else  
                  back_color = System.Drawing.Color.LemonChiffon;
              }                                                                           
              else
                if (severity == zx.matrix.log.CClient.SEVERITY_Error)
                {
                  if (aIsCurrentRow)
                    fore_color = System.Drawing.Color.Red;
                  else
                    back_color = System.Drawing.Color.PeachPuff;
                }  
                else
                  if (severity != zx.matrix.log.CClient.SEVERITY_Info)
                  {
                    // Unknown 
                    if (aIsCurrentRow)
                      fore_color = System.Drawing.Color.Cyan;
                    else
                      back_color = System.Drawing.Color.LightGray;
                  }  
            return new zx.Paint.CBrushSet(
               fore_color.IsEmpty ? ((System.Drawing.SolidBrush)aBrush.ForeBrush).Color : fore_color,
               back_color.IsEmpty ? ((System.Drawing.SolidBrush)aBrush.BackBrush).Color : back_color);
          }
          //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
          
          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        public  
          override void PaintCell(zx.Paint.CGridPainter.CColStyle aCell, 
                                      Graphics g, Rectangle bounds, CurrencyManager source, 
                                         int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
          {
            // Only for state column
            if (aCell.MappingName == "Severity")
            {
              // Be sure that record is in the buffer
              SupplyRecord(rowNum);
              // Select icon according to the state
              string severity = mDataGrid[rowNum, 0].ToString(); // the first field
              Icon icon;
              if (severity == zx.matrix.log.CClient.SEVERITY_Info)
                icon = mIconInfo;
              else
                if (severity == zx.matrix.log.CClient.SEVERITY_Success)  
                  icon = mIconSuccess;
                else  
                  if (severity == zx.matrix.log.CClient.SEVERITY_Warning)
                    icon = mIconWarning;
                  else
                    if (severity == zx.matrix.log.CClient.SEVERITY_Error)
                      icon = mIconError;
                    else
                      icon = mIconUnknown;  

              // Text area - call original base method, shifted right making place for the icon
              Rectangle bbtext = new Rectangle(bounds.X + 17, bounds.Y, bounds.Width - 17, bounds.Height);
              base.PaintCell(aCell, g, bbtext, source, rowNum, backBrush, foreBrush, alignToRight);
              // Drive icon in the beginning 
              Rectangle bb = new Rectangle(bounds.X + 1, bounds.Y, 16, bounds.Height);
              g.FillRectangle(backBrush, bounds.X, bounds.Y, 17, bounds.Height); // fill rect by bkgr color
              g.DrawIcon(icon, bb); // draw icon
            } // if
            else
              base.PaintCell(aCell, g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
          }                               
          //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
          

          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
          // This method provides a presense of requested record in the buffer
          // aRow must be in range!
          void SupplyRecord(int aRow)
          {
            lock (mTable)
            {
              DataRow row = mTable.Rows[aRow];
              if (row["ID"] == Convert.DBNull)
              {
                // Lock data source for fast adding
                int current_row = mDataGrid.CurrentRowIndex;
                // To restore position of current line on the screen
                Rectangle old_rect = current_row >= 0 ? 
                                      mDataGrid.GetCurrentCellBounds() : new Rectangle(-1,-1,0,0);

                mDataGrid.SuspendLayout();
                mDataGrid.DataSource = null;
                // Record is not in the buffer - request to the log server
                // Calculate a requested range
                const int _range = 500;
                int first_rec = aRow;
                int last_rec = aRow;
                bool stop_move_first = false;
                bool stop_move_last = false;
                while (last_rec - first_rec < _range && !(stop_move_first && stop_move_last))
                {
                  if (!stop_move_first)
                    if (mTable.Rows[first_rec]["ID"] == Convert.DBNull)
                    {
                      first_rec--;
                      if (first_rec < 0)
                      {
                        first_rec = 0;
                        stop_move_first = true;
                      } // if
                    } // if
                    else
                    {
                      first_rec++;
                      stop_move_first = true;
                    } // else

                  if (!stop_move_last)
                    if (mTable.Rows[last_rec]["ID"] == Convert.DBNull)
                    {
                      last_rec++;
                      if (last_rec == mTable.Rows.Count)
                      {
                        last_rec--;
                        stop_move_last = true;
                      } // if
                    } // if
                    else
                    {
                      last_rec--;
                      stop_move_last = true;
                    } // else
                } // while
               
                // Fetch data 
                zx.CError err = null;
                mFormLog.SetConnectionStatus("Fetching...", 2);
                zx.pl.CPropertyList aRecord = mLogClient.Fetch(first_rec, last_rec - first_rec + 1, 
                                                                                              ref err);
              
                // Fill it
                mFormLog.FillData(first_rec, last_rec, aRecord);
                mFormLog.SetConnectionStatus("Connected", 0);
  
                // Restore the data source
                mDataGrid.DataSource = mTable;
//                mDataGrid.CurrentCell = new DataGridCell(current_row, 1);
//!!!! TO BE OPTIMIZED !!!!
            DataGridCell cell = new DataGridCell(current_row, 1);
            mDataGrid.CurrentCell = cell;
                
            if (old_rect.Y >= 0)
            {
              // Restore position on the screen
              Rectangle rec = mDataGrid.GetCurrentCellBounds();
              DataGridCell newcell = 
                               new DataGridCell(cell.RowNumber + (rec.Y - old_rect.Y)/(rec.Height+1), 1);
              if (newcell.RowNumber >= mTable.Rows.Count)
                newcell.RowNumber = mTable.Rows.Count - 1;
              mDataGrid.CurrentCell = newcell;
              mDataGrid.CurrentCell = cell;
            } // if
//!!!!
                
                mDataGrid.ResumeLayout();
              } // if
            } // lock
          }
          //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


        public
          CLogGrid(DataGrid aDataGrid, ImageList aImages, System.Data.DataTable aTable,
                   zx.matrix.log.CClient aLogClient, FormLog aFormLog) : base(aDataGrid)
          {
            mImages = aImages; 
            mTable = aTable;
            mLogClient = aLogClient;
            mFormLog = aFormLog;
            // Set icons
		    mIconInfo    = Icon.FromHandle(((Bitmap)aImages.Images[0]).GetHicon());
		    mIconSuccess = Icon.FromHandle(((Bitmap)aImages.Images[1]).GetHicon());
		    mIconWarning = Icon.FromHandle(((Bitmap)aImages.Images[2]).GetHicon());
		    mIconError   = Icon.FromHandle(((Bitmap)aImages.Images[3]).GetHicon());
		    mIconUnknown = Icon.FromHandle(((Bitmap)aImages.Images[4]).GetHicon());
             
            // Set colors 
            NormalRowBrushSet = new zx.Paint.CBrushSet(SystemColors.WindowText, SystemColors.Window);
            NormalRowBrushSet2 = NormalRowBrushSet;
            CurrentRowBrushSet = new zx.Paint.CBrushSet(SystemColors.ActiveCaptionText, SystemColors.ActiveCaption);
          }  
		}; // CComponentGrid
		//----------------------------------------------------------------------------
		

		public FormLog()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
  
            SetConnectionStatus("Disconnected", 1);
			
			mLogClient.SetEventNotification(new zx.matrix.log.EventNotification(EventNotification), this);

            // Set header            
            Text += " " + (mIndexInstance.Index + 1).ToString();

            // Attach a grid painter to the dataGrid
            zx.Paint.CGridPainter gp = new CLogGrid(dataGridLog, imageListSeverities, 
                                                                        dataTableLog, mLogClient, this);

            // Locate form according to ControlSaver
            new zx.CPositionSaver(Text, this, FormMain.ControlSaver);
            // Locate grid columns
            gp.Register(Text, FormMain.ControlSaver);
            
            // Subscribe for events
            mId_CEventComponentComponentStateChanged = 
              zx.matrix.CMatrixSubsystem.Subscribe(
                new zx.matrix.CEventComponentComponentStateChanged(), 
                new zx.matrix.ProcessHandle(Handler_CEventComponentComponentStateChanged),
                zx.matrix.CMatrixSubsystem.ESubscriptionType.Event);
		}
		
    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    void Handler_CEventComponentComponentStateChanged(zx.matrix.CMessage aMsg, zx.matrix.CAddress aSender)
    {
      zx.matrix.CEventComponentComponentStateChanged msg = 
                                                new zx.matrix.CEventComponentComponentStateChanged(aMsg);
      msg.Deserialize();
      if (msg.ComponentDesc.Name == LOG_SERVER_NAME)
        if (msg.NewState == (int)zx.matrix.TComponentState.STATE_INITIALIZED)
        {
          if (!mLogClient.IsOpen())
            OpenSession();
        } // if
        else 
        { 
          mLogClient.CloseSession();
          SetConnectionStatus("Connection lost", 1);
        } // else  
    }
    //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    // Notification message handler
    void EventNotification(long aRecordsNumber, zx.pl.CPropertyList aRecord)
    {
      lock (dataTableLog)
      {
        if (dataTableLog.Rows.Count != aRecordsNumber)
        {
          // Number of record has changed: adjust the data table

          if (aRecordsNumber < 0)
            aRecordsNumber = 0; // Probably a bad message!

          int current_row = dataGridLog.CurrentRowIndex;
          bool last_row = current_row == dataTableLog.Rows.Count - 1;
          // To restore position of current line on the screen
          Rectangle old_rect = current_row >= 0 ? 
                                      dataGridLog.GetCurrentCellBounds() : new Rectangle(-1,-1,0,0);
          
          dataGridLog.SuspendLayout();

          // Lock the data source to fill the table fast
          dataGridLog.DataSource = null;
          
          // Add dummy records if necessary
          while (dataTableLog.Rows.Count < aRecordsNumber)
          {
            DataRow row = dataTableLog.NewRow();
            row[dataColumnRecordID] = ++mRecordID;
            row[dataColumnSeverity] = zx.matrix.log.CClient.SEVERITY_Unknown;
            dataTableLog.Rows.Add(row);
          } // while
          
          // Assign values to the new records
          if (aRecord.Count() > 0)
          {
            int index = dataTableLog.Rows.Count - aRecord.Count();
            FillData(index, dataTableLog.Rows.Count - 1, aRecord);
          } // if
          
          if (dataTableLog.Rows.Count > aRecordsNumber)
          {
            // Records were removed on the server side. Remove old records by a strange way below. It is
            // much faster than using removing the first row in a loop (checked!)
            // Get id if the last record to be kept in the table
            int idlast = Convert.ToInt32(dataTableLog.Rows[0][dataColumnRecordID]) + 
                                                           dataTableLog.Rows.Count - (int)aRecordsNumber;
            // Select rows up to the last record                                                           
            DataRow[] rows = dataTableLog.Select("RecordID > '" + idlast.ToString() + "'");
            // Clear the table and add these records
            dataTableLog.Clear();
            for (int i = 0; i < rows.Length; i++)
            {
              DataRow row = rows[i];
              row[dataColumnRecordID] = idlast + i + 1;
              dataTableLog.Rows.Add(rows[i]);
            } // for  
          } // if  

          dataGridLog.DataSource = dataTableLog;
          
          if (dataTableLog.Rows.Count > 0)
          {
            DataGridCell cell = new DataGridCell(last_row ? dataTableLog.Rows.Count - 1 : current_row, 1);
            dataGridLog.CurrentCell = cell;
            
            if (old_rect.Y >= 0)
            {
              // Restore position on the screen
              Rectangle rec = dataGridLog.GetCurrentCellBounds();
              DataGridCell newcell = 
                               new DataGridCell(cell.RowNumber + (rec.Y - old_rect.Y)/(rec.Height+1), 1);
              if (newcell.RowNumber >= dataTableLog.Rows.Count)
                newcell.RowNumber = dataTableLog.Rows.Count - 1;
              dataGridLog.CurrentCell = newcell;
              dataGridLog.CurrentCell = cell;
            } // if
          }

          dataGridLog.ResumeLayout();                 
        } // if
        statusBarPanelEvents.Text = dataTableLog.Rows.Count.ToString() + " events";  
      } // lock
    }
    //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    // Fills data in data table from aRecords. 
    // If everything is Ok, aRecords.Count must be aLastRecord - aFirstRecord
    public void FillData(int aFirstRecord, int aLastRecord, zx.pl.CPropertyList aRecords)
    {
      lock (dataGridLog)
      {
        // Assign values to the new records
        int index = aFirstRecord;
        IDictionaryEnumerator rec_enum = aRecords.GetEnum();
        while (rec_enum.MoveNext() && index < dataTableLog.Rows.Count)
        {
          zx.pl.CPropertyList rec = (zx.pl.CPropertyList)rec_enum.Value;
          DataRow row = dataTableLog.Rows[index++];
          row[dataColumnHost] = 
                  rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageHost).DC.GetStringValue();
          row[dataColumnComponent] = 
                rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageComponent).DC.GetStringValue();
          int severity = 
                   rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageSeverity).DC.GetLongValue();
          if (severity >= (int)zx.matrix.log.TSeverities.LOG_INFO && 
              severity <= (int)zx.matrix.log.TSeverities.LOG_ERROR)
            row[dataColumnSeverity] = zx.matrix.log.CClient.SeverityNames[severity];
          row[dataColumnID] = 
              rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageID).DC.GetLongValue();
          row[dataColumnText] = 
              rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageText).DC.GetStringValue();
          row[dataColumnSession] = 
              rec.FindPropertyV(zx.matrix.log.CClient.LOGFLD_MessageComponentSession).DC.GetStringValue();
          row[dataColumnTime] = 
                zx.Functions.DateToString(rec.FindPropertyV(
                                    zx.matrix.log.CClient.LOGFLD_MessageTime).DC.GetDateTimeValue());
        } // while  
          
        if (index < aLastRecord)
          // Some records left unfilled
          for (int i = index; i <= aLastRecord && i < dataTableLog.Rows.Count; i++)
            dataTableLog.Rows[i][dataColumnID] = 0;
      } // lock  
    }
    //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
			  // Free the index object
			  mIndexInstance.Dispose();

				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FormLog));
            this.panelFilter = new System.Windows.Forms.Panel();
            this.panelFilterAll = new System.Windows.Forms.Panel();
            this.label4 = new System.Windows.Forms.Label();
            this.checkBoxSeverityError = new System.Windows.Forms.CheckBox();
            this.checkBoxSeverityWarning = new System.Windows.Forms.CheckBox();
            this.checkBoxSeverityInfo = new System.Windows.Forms.CheckBox();
            this.checkBoxSeveritySuccess = new System.Windows.Forms.CheckBox();
            this.textBoxFilterMessageIDs = new System.Windows.Forms.TextBox();
            this.label3 = new System.Windows.Forms.Label();
            this.textBoxFilterHosts = new System.Windows.Forms.TextBox();
            this.textBoxFilterComponents = new System.Windows.Forms.TextBox();
            this.label2 = new System.Windows.Forms.Label();
            this.label1 = new System.Windows.Forms.Label();
            this.panelFilterButtons = new System.Windows.Forms.Panel();
            this.buttonRefresh = new System.Windows.Forms.Button();
            this.dataSetLog = new System.Data.DataSet();
            this.dataTableLog = new System.Data.DataTable();
            this.dataColumnSeverity = new System.Data.DataColumn();
            this.dataColumnTime = new System.Data.DataColumn();
            this.dataColumnID = new System.Data.DataColumn();
            this.dataColumnHost = new System.Data.DataColumn();
            this.dataColumnComponent = new System.Data.DataColumn();
            this.dataColumnSession = new System.Data.DataColumn();
            this.dataColumnText = new System.Data.DataColumn();
            this.dataColumnRecordID = new System.Data.DataColumn();
            this.imageListSeverities = new System.Windows.Forms.ImageList(this.components);
            this.panelTotal = new System.Windows.Forms.Panel();
            this.textBoxLog = new System.Windows.Forms.TextBox();
            this.statusBarLog = new System.Windows.Forms.StatusBar();
            this.statusBarPanelStatus = new System.Windows.Forms.StatusBarPanel();
            this.statusBarPanelEvents = new System.Windows.Forms.StatusBarPanel();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.dataGridLog = new System.Windows.Forms.DataGrid();
            this.imageListStates = new System.Windows.Forms.ImageList(this.components);
            this.panelFilter.SuspendLayout();
            this.panelFilterAll.SuspendLayout();
            this.panelFilterButtons.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dataSetLog)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataTableLog)).BeginInit();
            this.panelTotal.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelStatus)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelEvents)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridLog)).BeginInit();
            this.SuspendLayout();
            // 
            // panelFilter
            // 
            this.panelFilter.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelFilter.Controls.Add(this.panelFilterAll);
            this.panelFilter.Controls.Add(this.panelFilterButtons);
            this.panelFilter.Dock = System.Windows.Forms.DockStyle.Top;
            this.panelFilter.Location = new System.Drawing.Point(0, 0);
            this.panelFilter.Name = "panelFilter";
            this.panelFilter.Size = new System.Drawing.Size(640, 104);
            this.panelFilter.TabIndex = 0;
            // 
            // panelFilterAll
            // 
            this.panelFilterAll.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelFilterAll.Controls.Add(this.label4);
            this.panelFilterAll.Controls.Add(this.checkBoxSeverityError);
            this.panelFilterAll.Controls.Add(this.checkBoxSeverityWarning);
            this.panelFilterAll.Controls.Add(this.checkBoxSeverityInfo);
            this.panelFilterAll.Controls.Add(this.checkBoxSeveritySuccess);
            this.panelFilterAll.Controls.Add(this.textBoxFilterMessageIDs);
            this.panelFilterAll.Controls.Add(this.label3);
            this.panelFilterAll.Controls.Add(this.textBoxFilterHosts);
            this.panelFilterAll.Controls.Add(this.textBoxFilterComponents);
            this.panelFilterAll.Controls.Add(this.label2);
            this.panelFilterAll.Controls.Add(this.label1);
            this.panelFilterAll.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelFilterAll.Location = new System.Drawing.Point(0, 0);
            this.panelFilterAll.Name = "panelFilterAll";
            this.panelFilterAll.Size = new System.Drawing.Size(580, 100);
            this.panelFilterAll.TabIndex = 3;
            // 
            // label4
            // 
            this.label4.Location = new System.Drawing.Point(8, 72);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(72, 16);
            this.label4.TabIndex = 10;
            this.label4.Text = "Severities";
            // 
            // checkBoxSeverityError
            // 
            this.checkBoxSeverityError.Checked = true;
            this.checkBoxSeverityError.CheckState = System.Windows.Forms.CheckState.Checked;
            this.checkBoxSeverityError.Location = new System.Drawing.Point(326, 72);
            this.checkBoxSeverityError.Name = "checkBoxSeverityError";
            this.checkBoxSeverityError.Size = new System.Drawing.Size(72, 16);
            this.checkBoxSeverityError.TabIndex = 9;
            this.checkBoxSeverityError.Text = "Error";
            // 
            // checkBoxSeverityWarning
            // 
            this.checkBoxSeverityWarning.Checked = true;
            this.checkBoxSeverityWarning.CheckState = System.Windows.Forms.CheckState.Checked;
            this.checkBoxSeverityWarning.Location = new System.Drawing.Point(248, 72);
            this.checkBoxSeverityWarning.Name = "checkBoxSeverityWarning";
            this.checkBoxSeverityWarning.Size = new System.Drawing.Size(71, 16);
            this.checkBoxSeverityWarning.TabIndex = 8;
            this.checkBoxSeverityWarning.Text = "Warning";
            // 
            // checkBoxSeverityInfo
            // 
            this.checkBoxSeverityInfo.Checked = true;
            this.checkBoxSeverityInfo.CheckState = System.Windows.Forms.CheckState.Checked;
            this.checkBoxSeverityInfo.Location = new System.Drawing.Point(168, 72);
            this.checkBoxSeverityInfo.Name = "checkBoxSeverityInfo";
            this.checkBoxSeverityInfo.Size = new System.Drawing.Size(72, 16);
            this.checkBoxSeverityInfo.TabIndex = 7;
            this.checkBoxSeverityInfo.Text = "Info";
            // 
            // checkBoxSeveritySuccess
            // 
            this.checkBoxSeveritySuccess.Checked = true;
            this.checkBoxSeveritySuccess.CheckState = System.Windows.Forms.CheckState.Checked;
            this.checkBoxSeveritySuccess.Location = new System.Drawing.Point(88, 72);
            this.checkBoxSeveritySuccess.Name = "checkBoxSeveritySuccess";
            this.checkBoxSeveritySuccess.Size = new System.Drawing.Size(72, 16);
            this.checkBoxSeveritySuccess.TabIndex = 6;
            this.checkBoxSeveritySuccess.Text = "Success";
            // 
            // textBoxFilterMessageIDs
            // 
            this.textBoxFilterMessageIDs.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this.textBoxFilterMessageIDs.Location = new System.Drawing.Point(88, 50);
            this.textBoxFilterMessageIDs.Name = "textBoxFilterMessageIDs";
            this.textBoxFilterMessageIDs.Size = new System.Drawing.Size(481, 20);
            this.textBoxFilterMessageIDs.TabIndex = 5;
            this.textBoxFilterMessageIDs.Text = "";
            // 
            // label3
            // 
            this.label3.Location = new System.Drawing.Point(8, 50);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(72, 16);
            this.label3.TabIndex = 4;
            this.label3.Text = "Message IDs";
            // 
            // textBoxFilterHosts
            // 
            this.textBoxFilterHosts.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this.textBoxFilterHosts.Location = new System.Drawing.Point(88, 28);
            this.textBoxFilterHosts.Name = "textBoxFilterHosts";
            this.textBoxFilterHosts.Size = new System.Drawing.Size(480, 20);
            this.textBoxFilterHosts.TabIndex = 3;
            this.textBoxFilterHosts.Text = "";
            // 
            // textBoxFilterComponents
            // 
            this.textBoxFilterComponents.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this.textBoxFilterComponents.Location = new System.Drawing.Point(88, 5);
            this.textBoxFilterComponents.Name = "textBoxFilterComponents";
            this.textBoxFilterComponents.Size = new System.Drawing.Size(481, 20);
            this.textBoxFilterComponents.TabIndex = 2;
            this.textBoxFilterComponents.Text = "";
            // 
            // label2
            // 
            this.label2.Location = new System.Drawing.Point(8, 28);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(72, 16);
            this.label2.TabIndex = 1;
            this.label2.Text = "Hosts";
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(8, 8);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(72, 16);
            this.label1.TabIndex = 0;
            this.label1.Text = "Components";
            // 
            // panelFilterButtons
            // 
            this.panelFilterButtons.Controls.Add(this.buttonRefresh);
            this.panelFilterButtons.Dock = System.Windows.Forms.DockStyle.Right;
            this.panelFilterButtons.Location = new System.Drawing.Point(580, 0);
            this.panelFilterButtons.Name = "panelFilterButtons";
            this.panelFilterButtons.Size = new System.Drawing.Size(56, 100);
            this.panelFilterButtons.TabIndex = 1;
            // 
            // buttonRefresh
            // 
            this.buttonRefresh.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonRefresh.Image = ((System.Drawing.Image)(resources.GetObject("buttonRefresh.Image")));
            this.buttonRefresh.Location = new System.Drawing.Point(18, 14);
            this.buttonRefresh.Name = "buttonRefresh";
            this.buttonRefresh.Size = new System.Drawing.Size(24, 23);
            this.buttonRefresh.TabIndex = 1;
            this.buttonRefresh.Click += new System.EventHandler(this.buttonRefresh_Click);
            // 
            // dataSetLog
            // 
            this.dataSetLog.DataSetName = "NewDataSet";
            this.dataSetLog.Locale = new System.Globalization.CultureInfo("de-DE");
            this.dataSetLog.Tables.AddRange(new System.Data.DataTable[] {
                                                                            this.dataTableLog});
            // 
            // dataTableLog
            // 
            this.dataTableLog.Columns.AddRange(new System.Data.DataColumn[] {
                                                                                this.dataColumnSeverity,
                                                                                this.dataColumnTime,
                                                                                this.dataColumnID,
                                                                                this.dataColumnHost,
                                                                                this.dataColumnComponent,
                                                                                this.dataColumnSession,
                                                                                this.dataColumnText,
                                                                                this.dataColumnRecordID});
            this.dataTableLog.TableName = "TableLog";
            // 
            // dataColumnSeverity
            // 
            this.dataColumnSeverity.Caption = "Severity";
            this.dataColumnSeverity.ColumnName = "Severity";
            // 
            // dataColumnTime
            // 
            this.dataColumnTime.Caption = "Time";
            this.dataColumnTime.ColumnName = "Time";
            // 
            // dataColumnID
            // 
            this.dataColumnID.Caption = "ID";
            this.dataColumnID.ColumnName = "ID";
            this.dataColumnID.DataType = typeof(int);
            // 
            // dataColumnHost
            // 
            this.dataColumnHost.Caption = "Host";
            this.dataColumnHost.ColumnName = "Host";
            // 
            // dataColumnComponent
            // 
            this.dataColumnComponent.Caption = "Component";
            this.dataColumnComponent.ColumnName = "Component";
            // 
            // dataColumnSession
            // 
            this.dataColumnSession.Caption = "Session";
            this.dataColumnSession.ColumnName = "Session";
            // 
            // dataColumnText
            // 
            this.dataColumnText.Caption = "Text";
            this.dataColumnText.ColumnName = "Text";
            // 
            // dataColumnRecordID
            // 
            this.dataColumnRecordID.Caption = "";
            this.dataColumnRecordID.ColumnName = "RecordID";
            this.dataColumnRecordID.DataType = typeof(int);
            // 
            // imageListSeverities
            // 
            this.imageListSeverities.ImageSize = new System.Drawing.Size(16, 16);
            this.imageListSeverities.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListSeverities.ImageStream")));
            this.imageListSeverities.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // panelTotal
            // 
            this.panelTotal.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelTotal.Controls.Add(this.textBoxLog);
            this.panelTotal.Controls.Add(this.statusBarLog);
            this.panelTotal.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.panelTotal.Location = new System.Drawing.Point(0, 261);
            this.panelTotal.Name = "panelTotal";
            this.panelTotal.Size = new System.Drawing.Size(640, 88);
            this.panelTotal.TabIndex = 2;
            // 
            // textBoxLog
            // 
            this.textBoxLog.BackColor = System.Drawing.SystemColors.ControlLight;
            this.textBoxLog.Dock = System.Windows.Forms.DockStyle.Fill;
            this.textBoxLog.Location = new System.Drawing.Point(0, 0);
            this.textBoxLog.Multiline = true;
            this.textBoxLog.Name = "textBoxLog";
            this.textBoxLog.ReadOnly = true;
            this.textBoxLog.Size = new System.Drawing.Size(636, 62);
            this.textBoxLog.TabIndex = 4;
            this.textBoxLog.Text = "";
            // 
            // statusBarLog
            // 
            this.statusBarLog.Location = new System.Drawing.Point(0, 62);
            this.statusBarLog.Name = "statusBarLog";
            this.statusBarLog.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
                                                                                            this.statusBarPanelStatus,
                                                                                            this.statusBarPanelEvents});
            this.statusBarLog.ShowPanels = true;
            this.statusBarLog.Size = new System.Drawing.Size(636, 22);
            this.statusBarLog.TabIndex = 3;
            // 
            // statusBarPanelStatus
            // 
            this.statusBarPanelStatus.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
            this.statusBarPanelStatus.Text = "Not connected";
            this.statusBarPanelStatus.Width = 87;
            // 
            // statusBarPanelEvents
            // 
            this.statusBarPanelEvents.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
            this.statusBarPanelEvents.Text = "0 events";
            this.statusBarPanelEvents.Width = 57;
            // 
            // splitter1
            // 
            this.splitter1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splitter1.Location = new System.Drawing.Point(0, 258);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(640, 3);
            this.splitter1.TabIndex = 3;
            this.splitter1.TabStop = false;
            // 
            // dataGridLog
            // 
            this.dataGridLog.AllowSorting = false;
            this.dataGridLog.CaptionVisible = false;
            this.dataGridLog.DataMember = "";
            this.dataGridLog.DataSource = this.dataTableLog;
            this.dataGridLog.Dock = System.Windows.Forms.DockStyle.Fill;
            this.dataGridLog.HeaderForeColor = System.Drawing.SystemColors.ControlText;
            this.dataGridLog.Location = new System.Drawing.Point(0, 104);
            this.dataGridLog.Name = "dataGridLog";
            this.dataGridLog.Size = new System.Drawing.Size(640, 154);
            this.dataGridLog.TabIndex = 4;
            this.dataGridLog.CurrentCellChanged += new System.EventHandler(this.dataGridLog_CurrentCellChanged);
            // 
            // imageListStates
            // 
            this.imageListStates.ImageSize = new System.Drawing.Size(16, 16);
            this.imageListStates.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListStates.ImageStream")));
            this.imageListStates.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // FormLog
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(640, 349);
            this.Controls.Add(this.dataGridLog);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.panelTotal);
            this.Controls.Add(this.panelFilter);
            this.MaximizeBox = false;
            this.Name = "FormLog";
            this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
            this.Text = "Log";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.FormLog_Closing);
            this.panelFilter.ResumeLayout(false);
            this.panelFilterAll.ResumeLayout(false);
            this.panelFilterButtons.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dataSetLog)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataTableLog)).EndInit();
            this.panelTotal.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelStatus)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelEvents)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridLog)).EndInit();
            this.ResumeLayout(false);

        }
		#endregion

        private void FormLog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
          // Unregister the session
          mLogClient.CloseSession();
        }

        private void buttonRefresh_Click(object sender, System.EventArgs e)
        {
          // Reopen session with current filter
          buttonRefresh.Enabled = false;
          SetConnectionStatus("Connecting...", 2);
          mLogClient.CloseSession();
          lock (dataTableLog)
            dataTableLog.Clear();
          OpenSession();
          buttonRefresh.Enabled = true;
        }

        private void dataGridLog_CurrentCellChanged(object sender, System.EventArgs e)
        {
          lock (dataTableLog)
          {
            DataRow row = dataTableLog.Rows[dataGridLog.CurrentCell.RowNumber];
            if (textBoxLog.Text != row[dataColumnText].ToString())
              textBoxLog.Text = row[dataColumnText].ToString();
          } // lock
        }
	}
}
