/******************************************************************************
File:        FormMain.cs
Project:     
Appointment: main form
Author:      AR
Last Change: 23.09.2021
*******************************************************************************/

using System;
using System.Drawing;
using System.Windows.Forms;
using zx.matrix;

namespace Monitor
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class FormMain : System.Windows.Forms.Form
	{
    private System.Windows.Forms.MenuItem menuItemFileExit;
    private System.Windows.Forms.MenuItem menuItemFile;
    private System.Windows.Forms.MenuItem menuItemWindow;
    private System.Windows.Forms.MenuItem menuItemWindowComponents;
    private System.ComponentModel.IContainer components;
    private System.Windows.Forms.ToolBar toolBarMain;
    private System.Windows.Forms.StatusBar statusBarMain;
    private System.Windows.Forms.StatusBarPanel statusBarPanelConnection;
    private System.Windows.Forms.ImageList imageListMain;

    // Image indexes
    const int IMAGE_LAMP_ON = 0;
    private System.Windows.Forms.MainMenu mainMenu;
    private System.Windows.Forms.ImageList imageListToolBar;
    private System.Windows.Forms.ToolBarButton toolBarButtonComponents;
    private System.Windows.Forms.MenuItem menuItemWindowCascade;
    private System.Windows.Forms.MenuItem menuItem3;
    private System.Windows.Forms.MenuItem menuItemWindowTileVertical;
    private System.Windows.Forms.MenuItem menuItemWindowTileHorizontal;
    const int IMAGE_LAMP_OFF = 1;
    private System.Windows.Forms.MenuItem menuItemWindowLog;
    private const string Version = "2.10";

    // Control saver
    public static zx.CControlSaver ControlSaver;

    //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    public FormMain()
    {
      // Required for Windows Form Designer support
      InitializeComponent();

      // Create a control saver
      ControlSaver = new zx.CControlSaver(
        this, System.IO.Path.Combine(zx.path.GetUserAppPath(), "DcnMonitor.ini"));
      // Locate form according to ControlSaver
      new zx.CPositionSaver("", this, ControlSaver);

      // Initialize the matrix
      CMatrixSubsystem.Start("ftMonitor", Version);

      // Display "dysconnected" in panel
      PanelSetDisconnected(); // We are not connected yet

      CMatrixSubsystem.Subscribe(
        new zx.matrix.CEventConnected(), 
        new CControlMethodMessageHandler(
          new ProcessHandle(WhenConnectedEvent), statusBarPanelConnection.Parent), 
        CMatrixSubsystem.ESubscriptionType.Event);
      CMatrixSubsystem.Subscribe(
        new zx.matrix.CEventDisconnected(), 
        new CControlMethodMessageHandler(
          new ProcessHandle(WhenDisconnectedEvent), statusBarPanelConnection.Parent), 
        CMatrixSubsystem.ESubscriptionType.Event);
    }

    //==========================================================================
    // Connected event handlers
    private void WhenConnectedEvent(zx.matrix.CMessage aMsg, zx.matrix.CAddress aAddr)
    {
	    statusBarPanelConnection.Text = "Connected to " + CMatrixSubsystem.GetMasterAddress().GetFullName();
	    statusBarPanelConnection.Icon = Icon.FromHandle(((Bitmap)imageListMain.Images[IMAGE_LAMP_ON]).GetHicon());
    }

    private void PanelSetDisconnected()
    {
      statusBarPanelConnection.Text = "Disconnected";
      statusBarPanelConnection.Icon = Icon.FromHandle(((Bitmap)imageListMain.Images[IMAGE_LAMP_OFF]).GetHicon());
    }
    private void WhenDisconnectedEvent(zx.matrix.CMessage aMsg, zx.matrix.CAddress aAddr)
    {
      PanelSetDisconnected();
    }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
                  components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FormMain));
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.menuItemFile = new System.Windows.Forms.MenuItem();
            this.menuItemFileExit = new System.Windows.Forms.MenuItem();
            this.menuItemWindow = new System.Windows.Forms.MenuItem();
            this.menuItemWindowTileHorizontal = new System.Windows.Forms.MenuItem();
            this.menuItemWindowTileVertical = new System.Windows.Forms.MenuItem();
            this.menuItemWindowCascade = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItemWindowComponents = new System.Windows.Forms.MenuItem();
            this.menuItemWindowLog = new System.Windows.Forms.MenuItem();
            this.toolBarMain = new System.Windows.Forms.ToolBar();
            this.toolBarButtonComponents = new System.Windows.Forms.ToolBarButton();
            this.imageListToolBar = new System.Windows.Forms.ImageList(this.components);
            this.statusBarMain = new System.Windows.Forms.StatusBar();
            this.statusBarPanelConnection = new System.Windows.Forms.StatusBarPanel();
            this.imageListMain = new System.Windows.Forms.ImageList(this.components);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelConnection)).BeginInit();
            this.SuspendLayout();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemFile,
            this.menuItemWindow});
            // 
            // menuItemFile
            // 
            this.menuItemFile.Index = 0;
            this.menuItemFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemFileExit});
            this.menuItemFile.Text = "&File";
            // 
            // menuItemFileExit
            // 
            this.menuItemFileExit.Index = 0;
            this.menuItemFileExit.Text = "Exit";
            this.menuItemFileExit.Click += new System.EventHandler(this.menuItemFileExit_Click);
            // 
            // menuItemWindow
            // 
            this.menuItemWindow.Index = 1;
            this.menuItemWindow.MdiList = true;
            this.menuItemWindow.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemWindowTileHorizontal,
            this.menuItemWindowTileVertical,
            this.menuItemWindowCascade,
            this.menuItem3,
            this.menuItemWindowComponents,
            this.menuItemWindowLog});
            this.menuItemWindow.Text = "&Window";
            // 
            // menuItemWindowTileHorizontal
            // 
            this.menuItemWindowTileHorizontal.Index = 0;
            this.menuItemWindowTileHorizontal.Text = "Tile &horizontal";
            this.menuItemWindowTileHorizontal.Click += new System.EventHandler(this.menuItemWindowTileHorizontal_Click);
            // 
            // menuItemWindowTileVertical
            // 
            this.menuItemWindowTileVertical.Index = 1;
            this.menuItemWindowTileVertical.Text = "Tile &vertical";
            this.menuItemWindowTileVertical.Click += new System.EventHandler(this.menuItemWindowTileVertical_Click);
            // 
            // menuItemWindowCascade
            // 
            this.menuItemWindowCascade.Index = 2;
            this.menuItemWindowCascade.Text = "&Cascade";
            this.menuItemWindowCascade.Click += new System.EventHandler(this.menuItemWindowCascade_Click);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 3;
            this.menuItem3.Text = "-";
            // 
            // menuItemWindowComponents
            // 
            this.menuItemWindowComponents.Index = 4;
            this.menuItemWindowComponents.Text = "&Components";
            this.menuItemWindowComponents.Click += new System.EventHandler(this.menuItemWindowComponents_Click);
            // 
            // menuItemWindowLog
            // 
            this.menuItemWindowLog.Index = 5;
            this.menuItemWindowLog.Text = "&Log";
            this.menuItemWindowLog.Click += new System.EventHandler(this.menuItemWindowLog_Click);
            // 
            // toolBarMain
            // 
            this.toolBarMain.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
            this.toolBarMain.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
            this.toolBarButtonComponents});
            this.toolBarMain.ButtonSize = new System.Drawing.Size(24, 24);
            this.toolBarMain.DropDownArrows = true;
            this.toolBarMain.ImageList = this.imageListToolBar;
            this.toolBarMain.Location = new System.Drawing.Point(0, 0);
            this.toolBarMain.Name = "toolBarMain";
            this.toolBarMain.ShowToolTips = true;
            this.toolBarMain.Size = new System.Drawing.Size(424, 28);
            this.toolBarMain.TabIndex = 1;
            this.toolBarMain.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBarMain_ButtonClick);
            // 
            // toolBarButtonComponents
            // 
            this.toolBarButtonComponents.ImageIndex = 0;
            this.toolBarButtonComponents.Name = "toolBarButtonComponents";
            this.toolBarButtonComponents.ToolTipText = "List of Components";
            // 
            // imageListToolBar
            // 
            this.imageListToolBar.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListToolBar.ImageStream")));
            this.imageListToolBar.TransparentColor = System.Drawing.Color.Transparent;
            this.imageListToolBar.Images.SetKeyName(0, "");
            // 
            // statusBarMain
            // 
            this.statusBarMain.Location = new System.Drawing.Point(0, 383);
            this.statusBarMain.Name = "statusBarMain";
            this.statusBarMain.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
            this.statusBarPanelConnection});
            this.statusBarMain.ShowPanels = true;
            this.statusBarMain.Size = new System.Drawing.Size(424, 22);
            this.statusBarMain.TabIndex = 2;
            // 
            // statusBarPanelConnection
            // 
            this.statusBarPanelConnection.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
            this.statusBarPanelConnection.Name = "statusBarPanelConnection";
            this.statusBarPanelConnection.Width = 10;
            // 
            // imageListMain
            // 
            this.imageListMain.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListMain.ImageStream")));
            this.imageListMain.TransparentColor = System.Drawing.Color.Transparent;
            this.imageListMain.Images.SetKeyName(0, "");
            this.imageListMain.Images.SetKeyName(1, "");
            // 
            // FormMain
            // 
            this.ClientSize = new System.Drawing.Size(424, 405);
            this.Controls.Add(this.statusBarMain);
            this.Controls.Add(this.toolBarMain);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.IsMdiContainer = true;
            this.Menu = this.mainMenu;
            this.Name = "FormMain";
            this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
            this.Text = "ftMonitor (version " + Version + ")";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.FormMain_Closing);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelConnection)).EndInit();
            this.ResumeLayout(false);
            this.PerformLayout();

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
          Application.Run(new FormMain());
		}

        private void menuItemFileExit_Click(object sender, System.EventArgs e)
        {
            Close();
        }

        private void menuItemWindowComponents_Click(object sender, System.EventArgs e)
        {
          FormComponents f = new FormComponents();
          f.MdiParent = this;
          f.Show();
        }

        private void toolBarMain_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
        {
          if (e.Button == toolBarButtonComponents)
          {
            // Create a new components window
            menuItemWindowComponents_Click(sender, new System.EventArgs());
          }
        }

        private void menuItemWindowTileHorizontal_Click(object sender, System.EventArgs e)
        {
            LayoutMdi(System.Windows.Forms.MdiLayout.TileHorizontal);
        }

        private void menuItemWindowTileVertical_Click(object sender, System.EventArgs e)
        {
            LayoutMdi(System.Windows.Forms.MdiLayout.TileVertical);
        }

        private void menuItemWindowCascade_Click(object sender, System.EventArgs e)
        {
            LayoutMdi(System.Windows.Forms.MdiLayout.Cascade);
        }

        private void FormMain_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Stop the matrix (seems must be stopped here, not in Dispose()...)
            CMatrixSubsystem.Stop();
        }

        private void menuItemWindowLog_Click(object sender, System.EventArgs e)
        {
          FormLog f = new FormLog();
          f.MdiParent = this;
          f.Show();
        }
	}
}
